#include "PDTesterAPI.h"
	
#define GET_DEV_INFO					0x01
#define GET_CONSTAT						0x0A
#define GET_PORT_CAPABILITIES			0x0B
#define GET_STAT						0x0C
#define SET_PORT_VOLTAGE				0x0D
#define SET_DEF_VOLTAGE					0x0E
#define SET_DEF_CURRENT					0x0F

#define	SET_CURRENT						0x10
#define SET_DEF_PROFILE 				0x12

#define SET_CALIB_DATA					0xE3
#define GET_CALIB_DATA					0xE4
#define RESET_CALIB_DATA				0xE5

#define EVENT_PORT_ATTACHED				0x20
#define EVENT_PORT_DETACHED				0x21
#define EVENT_PROFILE_CHANGED			0x22
#define EVENT_NEW_CAPABILITY			0x23
#define	SET_CONFIG						0xE0
#define	GET_CONFIG						0xE1

#define	CFG_LOOPBACK_ENABLE			0
#define	CFG_SET_MAX_CURRENT			1
#define	CFG_SDP_MAX_CURRENT			2
#define	CFG_ESTIMATE_VBUS			3
#define	CFG_CABLE_RESISTANCE		4

#define	TIME_OUT_MS					500

void SerialEventManager(void* object, void* parent_class, uint32 event)
{
	PDTester *self = static_cast<PDTester*>(parent_class);
	Tserial_event *com;
	com = (Tserial_event *)object;

	self->onSerialEvent(object, event);
}

PDTester::PDTester()
{
	int i;

	RcvDataLen = 0;
	NumRcvBytes = 0;
	NumRemaingBytes = 1;

	// creating Events for the different sources
	for (i = 0; i<COM_SIGNAL_NBR; i++)
		com_events[i] = CreateEvent(NULL, FALSE, FALSE, NULL);
}

PDTester::~PDTester()
{
	int i;

	for (i = 0; i<COM_SIGNAL_NBR; i++)         // deleting the events
	{
		if (com_events[i] != INVALID_HANDLE_VALUE)
			CloseHandle(com_events[i]);
		com_events[i] = INVALID_HANDLE_VALUE;
	}
}

BOOL PDTester::Connect(char *port, type_EventCallBack event_callback)
{
	BOOL status = FALSE;

	InitializeCriticalSection(&DeviceCritSection);
	
	com = new Tserial_event();
	if (com != 0)
	{
		EventCallback = event_callback;
		com->setManager(SerialEventManager, this);
		com->setRxSize(SERIAL_MAX_RX);
		status = com->connect(port);
		if (status == 0)
		{
			com->setRxSize(1);
			com->dataHasBeenRead();
			return FALSE;
		}
	}

	if (!this->GetDevInfo(&HW_Ver, &FW_Ver))
		return FALSE;
	this->GetConfig();
	
	return TRUE;
}

void PDTester::Disconnect()
{
	
	if (com != 0)
	{
		com->disconnect();
	}
}

/* ======================================================== */
/* ===============  OnCharArrival     ===================== */
/* ======================================================== */
void PDTester::onSerialEvent(void *object, uint32 event)
{
	char	*ptr;
	int		size;
	int		i;
	bool	bPacketReceived = 0;
	Tserial_event *com;

	com = (Tserial_event *)object;
	if (com != 0)
	{
		switch (event)
		{
		case  SERIAL_CONNECTED:
			com->dataHasBeenRead();
			com->setRxSize(1);
			break;
		case  SERIAL_DISCONNECTED:
			break;
		case  SERIAL_DATA_SENT:
			SetEvent(com_events[COM_PACKET_SENT]);
			break;
		case  SERIAL_RING:
			break;
		case  SERIAL_CD_ON:
			break;
		case  SERIAL_CD_OFF:
			break;
		case  SERIAL_DATA_ARRIVAL:
			size = com->getDataInSize();
			ptr = com->getDataInBuffer();
			if (size == 0)
			{
				RcvDataLen = 0;
				NumRemaingBytes = 1;
			}
			for (i = 0; i < size; i++)
			{
				if (NumRcvBytes == 0)
				{
					if (*ptr != 0x02)
						RcvDataLen = 0;
					else
						ComRcvBuffer[NumRcvBytes++] = *ptr;
					NumRemaingBytes = 1;
				}
				else if (NumRcvBytes == 1)
				{
					ComRcvBuffer[NumRcvBytes++] = *ptr;
					RcvDataLen = *ptr;
					NumRemaingBytes = RcvDataLen + 2;
				}
				else
				{
					NumRemaingBytes--;
					ComRcvBuffer[NumRcvBytes++] = *ptr;
					if (!NumRemaingBytes)
					{
						if (*ptr == 0x03)
						{
							memcpy(RcvData, &ComRcvBuffer[2], RcvDataLen);
							bPacketReceived = 1;
							NumPackets++;
						}
						else
							RcvDataLen = 0;
						NumRcvBytes = 0;
						NumRemaingBytes = 1;
					}
					
				}
				*ptr++;
			}
			com->setRxSize(NumRemaingBytes);
			com->dataHasBeenRead();
			if (bPacketReceived)
			{
				if (RcvData[0] == EVENT_PORT_ATTACHED)
					EventCallback(PDAPI_EVENT_PORT_ATTACHED);
				else if (RcvData[0] == EVENT_PORT_DETACHED)
					EventCallback(PDAPI_EVENT_PORT_DETACHED);
				else if (RcvData[0] == EVENT_PROFILE_CHANGED)
					EventCallback(PDAPI_EVENT_PROFILE_CHANGED);
				else if (RcvData[0] == EVENT_NEW_CAPABILITY)
					EventCallback(PDAPI_EVENT_NEW_CAPABILITY);
				else
					SetEvent(com_events[COM_PACKET_ARRIVED]);
			}
		}
	}
}

UCHAR checksum(UCHAR *buf)
{
	UCHAR chk = 0;
	int i;

	int DataLen = buf[1];
	buf[DataLen + 2] = 0;
	for (i = 0; i < DataLen + 2; i++)
		chk ^= buf[i];
	chk ^= 0x03;
	return chk;
}

//Get the serial numbers and device number of the currently connecetd plugs
//Return the number of plugs found or -1 for on error
int PDTester::GetConnectedDevices(char *devices[MAX_NUM_TESTERS])
{
	int numUSB3Plugsfound = 0;
	FT_STATUS		ftStatus;
	DWORD			devcount;

	ftStatus = FT_ListDevices(&devcount, NULL, FT_LIST_NUMBER_ONLY);

	char		serial[MAX_SERIAL_LENGTH];

	for (DWORD Index = 0; Index < devcount; Index++)
	{
		ftStatus = FT_ListDevices((PVOID)Index, serial, FT_LIST_BY_INDEX | FT_OPEN_BY_SERIAL_NUMBER);

		if (!strncmp(serial, USBPD_SERIAL_PREFIX, 4))
		{
			strcpy_s(devices[numUSB3Plugsfound], MAX_SERIAL_LENGTH, serial);
			numUSB3Plugsfound++;
		}
	}

	return  numUSB3Plugsfound;
}

BOOL PDTester::GetDevInfo(UCHAR *HW_Ver, UCHAR *SW_Ver)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x01;
	cBuf[2] = GET_DEV_INFO;
	cBuf[3] = checksum(cBuf);
	cBuf[4] = 0x03;
	com->sendData((char*)cBuf, 5);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			if (RcvData[0] == GET_DEV_INFO)
			{
				*HW_Ver = RcvData[1];
				*SW_Ver = RcvData[2];
			}
			else
				retVal = FALSE;
		}
		else
		{
			*HW_Ver = 10;
			*SW_Ver = 0;
			retVal = FALSE;
		}
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::GetConfig(void)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x02;
	cBuf[2] = GET_CONFIG;
	cBuf[3] = CFG_LOOPBACK_ENABLE;
	cBuf[4] = checksum(cBuf);
	cBuf[5] = 0x03;
	com->sendData((char*)cBuf, 6);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0 && RcvData[0] == GET_CONFIG)
		{
			bLoopbackPortEnabled = RcvData[1];
		}
		else
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	if (retVal == TRUE)
	{
		cBuf[0] = 0x02;
		cBuf[1] = 0x02;
		cBuf[2] = GET_CONFIG;
		cBuf[3] = CFG_SDP_MAX_CURRENT;
		cBuf[4] = checksum(cBuf);
		cBuf[5] = 0x03;
		com->sendData((char*)cBuf, 6);
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
			if (status == WAIT_OBJECT_0 && RcvData[0] == GET_CONFIG)
			{
				MaxSDPCurrent = ((unsigned int)RcvData[2] << 8) + RcvData[1];
			}
			else
				retVal = FALSE;
		}
		else
			retVal = FALSE;
	}

	if (retVal == TRUE)
	{
		cBuf[0] = 0x02;
		cBuf[1] = 0x02;
		cBuf[2] = GET_CONFIG;
		cBuf[3] = CFG_ESTIMATE_VBUS;
		cBuf[4] = checksum(cBuf);
		cBuf[5] = 0x03;
		com->sendData((char*)cBuf, 6);
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
			if (status == WAIT_OBJECT_0 && RcvData[0] == GET_CONFIG)
			{
				bEstimatePortVoltage = RcvData[1];
			}
			else
				retVal = FALSE;
		}
		else
			retVal = FALSE;
	}

	if (retVal == TRUE)
	{
		cBuf[0] = 0x02;
		cBuf[1] = 0x02;
		cBuf[2] = GET_CONFIG;
		cBuf[3] = CFG_CABLE_RESISTANCE;
		cBuf[4] = checksum(cBuf);
		cBuf[5] = 0x03;
		com->sendData((char*)cBuf, 6);
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0 && RcvData[0] == GET_CONFIG)
		{
			status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
			if (status == WAIT_OBJECT_0 && RcvData[0] == GET_CONFIG)
			{
				CableResistance = ((unsigned int)RcvData[2] << 8) + RcvData[1];
			}
			else
				retVal = FALSE;
		}
		else
			retVal = FALSE;
	}

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::SetConfig(void)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x03;
	cBuf[2] = SET_CONFIG;
	cBuf[3] = CFG_LOOPBACK_ENABLE;
	cBuf[4] = bLoopbackPortEnabled;
	cBuf[5] = checksum(cBuf);
	cBuf[6] = 0x03;
	com->sendData((char*)cBuf, 7);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status != WAIT_OBJECT_0 || RcvData[0] != SET_CONFIG)
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	Sleep(100);

	if (retVal == TRUE)
	{

		if (CurrentLimitType == FORCE_LIMIT)
		{
			cBuf[0] = 0x02;
			cBuf[1] = 0x05;
			cBuf[2] = SET_CONFIG;
			cBuf[3] = CFG_SET_MAX_CURRENT;
			cBuf[4] = CurrentLimitType;
			cBuf[5] = (UCHAR)MaxCurrent & 0xff;
			cBuf[6] = (UCHAR)((MaxCurrent >> 8) & 0xff);
			cBuf[7] = checksum(cBuf);
			cBuf[8] = 0x03;
			com->sendData((char*)cBuf, 9);
		}
		else
		{
			cBuf[0] = 0x02;
			cBuf[1] = 0x03;
			cBuf[2] = SET_CONFIG;
			cBuf[3] = CFG_SET_MAX_CURRENT;
			cBuf[4] = CurrentLimitType;
			cBuf[5] = checksum(cBuf);
			cBuf[6] = 0x03;
			com->sendData((char*)cBuf, 7);
		}
		
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
			if (status != WAIT_OBJECT_0 || RcvData[0] != SET_CONFIG)
				retVal = FALSE;
		}
		else
			retVal = FALSE;
	}

	Sleep(100);

	if (retVal == TRUE)
	{
		cBuf[0] = 0x02;
		cBuf[1] = 0x04;
		cBuf[2] = SET_CONFIG;
		cBuf[3] = CFG_SDP_MAX_CURRENT;
		cBuf[4] = (UCHAR)MaxSDPCurrent & 0xff;
		cBuf[5] = (UCHAR)(MaxSDPCurrent >> 8) & 0xff;
		cBuf[6] = checksum(cBuf);
		cBuf[7] = 0x03;
		com->sendData((char*)cBuf, 8);
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
			if (status != WAIT_OBJECT_0 || RcvData[0] != SET_CONFIG)
				retVal = FALSE;
		}
		else
			retVal = FALSE;
	}

	Sleep(100);

	if (retVal == TRUE)
	{
		cBuf[0] = 0x02;
		cBuf[1] = 0x03;
		cBuf[2] = SET_CONFIG;
		cBuf[3] = CFG_ESTIMATE_VBUS;
		cBuf[4] = bEstimatePortVoltage;
		cBuf[5] = checksum(cBuf);
		cBuf[6] = 0x03;
		com->sendData((char*)cBuf, 7);
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
			if (status != WAIT_OBJECT_0 || RcvData[0] != SET_CONFIG)
				retVal = FALSE;
		}
		else
			retVal = FALSE;
	}

	Sleep(100);

	if (retVal == TRUE)
	{
		cBuf[0] = 0x02;
		cBuf[1] = 0x04;
		cBuf[2] = SET_CONFIG;
		cBuf[3] = CFG_CABLE_RESISTANCE;
		cBuf[4] = (UCHAR)CableResistance & 0xff;
		cBuf[5] = (UCHAR)(CableResistance >> 8) & 0xff;
		cBuf[6] = checksum(cBuf);
		cBuf[7] = 0x03;
		com->sendData((char*)cBuf, 8);
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
			if (status != WAIT_OBJECT_0 || RcvData[0] != SET_CONFIG)
				retVal = FALSE;
		}
		else
			retVal = FALSE;
	}

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::SetCurrentLimit(void)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	if (CurrentLimitType == FORCE_LIMIT)
	{
		cBuf[0] = 0x02;
		cBuf[1] = 0x05;
		cBuf[2] = SET_CONFIG;
		cBuf[3] = CFG_SET_MAX_CURRENT;
		cBuf[4] = CurrentLimitType;
		cBuf[5] = (UCHAR)MaxCurrent & 0xff;
		cBuf[6] = (UCHAR)((MaxCurrent >> 8) & 0xff);
		cBuf[7] = checksum(cBuf);
		cBuf[8] = 0x03;
		com->sendData((char*)cBuf, 9);
	}
	else
	{
		cBuf[0] = 0x02;
		cBuf[1] = 0x03;
		cBuf[2] = SET_CONFIG;
		cBuf[3] = CFG_SET_MAX_CURRENT;
		cBuf[4] = CurrentLimitType;
		cBuf[5] = checksum(cBuf);
		cBuf[6] = 0x03;
		com->sendData((char*)cBuf, 7);
	}

	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status != WAIT_OBJECT_0 || RcvData[0] != SET_CONFIG)
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::GetConnectionStatus(USB_ConnectionStatus_t *port_status, BYTE* profile_index, PROFILE_TypeDef *profile, BYTE *profile_subtype, UINT16 *voltage, UINT16 *max_current, UINT32 *max_power)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);
	
	cBuf[0] = 0x02;
	cBuf[1] = 0x01;
	cBuf[2] = GET_CONSTAT;
	cBuf[3] = checksum(cBuf);
	cBuf[4] = 0x03;
	com->sendData((char*)cBuf, 5);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0 && RcvData[0] == GET_CONSTAT)
		{
			*port_status = (USB_ConnectionStatus_t)RcvData[1];
			*profile_index = (USB_ConnectionStatus_t)RcvData[2];
			*profile = (PROFILE_TypeDef)RcvData[3];
			*profile_subtype = RcvData[4];
			*voltage = ((WORD)RcvData[6] << 8) | RcvData[5];
			*max_current = ((WORD)RcvData[8] << 8) | RcvData[7];
			*max_power = ((WORD)RcvData[12] << 24) | ((WORD)RcvData[11] << 16) | ((WORD)RcvData[10] << 8) | RcvData[9];
		}
		else
			retVal = FALSE;
	}
	else
		retVal = FALSE;
	
	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::GetCapabilities(USBPD_Capabilities_TypeDef *SourceCapabilities)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x01;
	cBuf[2] = GET_PORT_CAPABILITIES;
	cBuf[3] = checksum(cBuf);
	cBuf[4] = 0x03;
	com->sendData((char*)cBuf, 5);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0 && RcvData[0] == GET_PORT_CAPABILITIES)
		{
			memcpy((UCHAR*)SourceCapabilities, &RcvData[1], sizeof(USBPD_Capabilities_TypeDef));
		}
		else
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}


BOOL PDTester::GetStatistics(UCHAR *temp, UINT16 *voltage, UINT16 *set_current, UINT16 *current, UINT16 *loopback_current)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x01;
	cBuf[2] = GET_STAT;
	cBuf[3] = checksum(cBuf);
	cBuf[4] = 0x03;
	com->sendData((char*)cBuf, 5);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0 && RcvData[0] == GET_STAT)
		{
			*temp = RcvData[2];
			*voltage = ((WORD)RcvData[4] << 8) | RcvData[3];
			*set_current = ((WORD)RcvData[6] << 8) | RcvData[5];
			*current = ((WORD)RcvData[8] << 8) | RcvData[7];
			*loopback_current = ((WORD)RcvData[10] << 8) | RcvData[9];
		}
		else
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::SetVoltage(UCHAR index, UINT16 voltage)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x04;
	cBuf[2] = SET_PORT_VOLTAGE;
	cBuf[3] = index;
	cBuf[4] = voltage & 0xff;
	cBuf[5] = (voltage >> 8) & 0xff;
	cBuf[6] = checksum(cBuf);
	cBuf[7] = 0x03;
	com->sendData((char*)cBuf, 8);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status != WAIT_OBJECT_0 || RcvData[0] != SET_PORT_VOLTAGE)
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::SetLoad(UINT16 set_current)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x03;
	cBuf[2] = SET_CURRENT;
	cBuf[3] = set_current & 0xff;
	cBuf[4] = (set_current >> 8) & 0xff;
	cBuf[5] = checksum(cBuf);
	cBuf[6] = 0x03;
	com->sendData((char*)cBuf, 7);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status != WAIT_OBJECT_0 || RcvData[0] != SET_CURRENT)
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::SetDefaultVoltage(UINT16 voltage_mv)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x03;
	cBuf[2] = SET_DEF_VOLTAGE;
	cBuf[3] = voltage_mv & 0xff;
	cBuf[4] = (voltage_mv >> 8) & 0xff;
	cBuf[5] = checksum(cBuf);
	cBuf[6] = 0x03;
	com->sendData((char*)cBuf, 7);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status != WAIT_OBJECT_0 || RcvData[0] != SET_DEF_VOLTAGE)
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::SetDefaultProfile(UINT8 profile_index)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x02;
	cBuf[2] = SET_DEF_PROFILE;
	cBuf[3] = profile_index;
	cBuf[4] = checksum(cBuf);
	cBuf[5] = 0x03;
	com->sendData((char*)cBuf, 6);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status != WAIT_OBJECT_0 || RcvData[0] != SET_DEF_PROFILE)
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::SetDefaultLoad(UINT16 current_ma)
{
	UCHAR	cBuf[10];
	DWORD	status;
	BOOL	retVal = TRUE;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x03;
	cBuf[2] = SET_DEF_CURRENT;
	cBuf[3] = current_ma & 0xff;
	cBuf[4] = (current_ma >> 8) & 0xff;
	cBuf[5] = checksum(cBuf);
	cBuf[6] = 0x03;
	com->sendData((char*)cBuf, 7);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status != WAIT_OBJECT_0 || RcvData[0] != SET_DEF_CURRENT)
			retVal = FALSE;
	}
	else
		retVal = FALSE;

	LeaveCriticalSection(&DeviceCritSection);

	return retVal;
}

BOOL PDTester::GetCalibrationData(CalibrationChannel_t channel, BOOL* isCalibrated, int* year, int* month, int* applied1, int* measured1, int* applied2, int* measured2)
{
	UCHAR	cBuf[10];
	long status;

	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x02;
	cBuf[2] = GET_CALIB_DATA;
	cBuf[3] = channel;
	cBuf[4] = checksum(cBuf);
	cBuf[5] = 0x03;
	com->sendData((char*)cBuf, 6);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			if (RcvData[0] == GET_CALIB_DATA)
			{
				*isCalibrated = RcvData[1];
				*year = RcvData[2] + 2020;
				*month = RcvData[3];
				*applied1 = (INT16)((RcvData[5] << 8) | RcvData[4]);
				*measured1 = (INT16)((RcvData[7] << 8) | RcvData[6]);
				*applied2 = (INT16)((RcvData[9] << 8) | RcvData[8]);
				*measured2 = (INT16)((RcvData[11] << 8) | RcvData[10]);
				LeaveCriticalSection(&DeviceCritSection);
				return TRUE;
			}
		}
	}
	LeaveCriticalSection(&DeviceCritSection);
	return FALSE;
}


BOOL PDTester::SetCalibrationData(CalibrationChannel_t channel, int year, int month, int applied1, int measured1, int applied2, int measured2)
{
	UCHAR	cBuf[32];
	long status;
	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 12;
	cBuf[2] = SET_CALIB_DATA;
	cBuf[3] = channel;
	cBuf[4] = year - 2020;
	cBuf[5] = month;
	cBuf[6] = (UINT16)applied1 & 0xff;
	cBuf[7] = ((UINT16)applied1 >> 8) & 0xff;
	cBuf[8] = (UINT16)measured1 & 0xff;
	cBuf[9] = ((UINT16)measured1 >> 8) & 0xff;
	cBuf[10] = (UINT16)applied2 & 0xff;
	cBuf[11] = ((UINT16)applied2 >> 8) & 0xff;
	cBuf[12] = (UINT16)measured2 & 0xff;
	cBuf[13] = ((UINT16)measured2 >> 8) & 0xff;
	cBuf[14] = checksum(cBuf);
	cBuf[15] = 0x03;
	com->sendData((char*)cBuf, 16);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			if (RcvData[0] == SET_CALIB_DATA)
			{
				LeaveCriticalSection(&DeviceCritSection);
				return TRUE;
			}
		}
	}
	LeaveCriticalSection(&DeviceCritSection);
	return FALSE;
}

BOOL PDTester::ResetCalibrationData(CalibrationChannel_t channel)
{
	UCHAR	cBuf[32];
	long status;
	EnterCriticalSection(&DeviceCritSection);

	cBuf[0] = 0x02;
	cBuf[1] = 0x02;
	cBuf[2] = RESET_CALIB_DATA;
	cBuf[3] = channel;
	cBuf[4] = checksum(cBuf);
	cBuf[5] = 0x03;
	com->sendData((char*)cBuf, 6);
	status = WaitForMultipleObjects(1, &com_events[COM_PACKET_SENT], FALSE, TIME_OUT_MS);
	if (status == WAIT_OBJECT_0)
	{
		status = WaitForMultipleObjects(1, &com_events[COM_PACKET_ARRIVED], FALSE, TIME_OUT_MS);
		if (status == WAIT_OBJECT_0)
		{
			if (RcvData[0] == RESET_CALIB_DATA)
			{
				LeaveCriticalSection(&DeviceCritSection);
				return TRUE;
			}
		}
	}
	LeaveCriticalSection(&DeviceCritSection);
	return FALSE;
}

