#ifndef PASSMARK_PDTESTER_H
#define PASSMARK_PDTESTER_H

#include <stdio.h>
#include <windows.h>
#include "tserial_event.h"

#define USBPD_SERIAL_PREFIX		"PMPD"

#define MAX_NUM_TESTERS			32
#define MAX_SERIAL_LENGTH		16
#define USBPD_MAX_NB_PDO        (7U)              /*!< Maximum number of supported Power Data Objects: fix by the Specification */

#define COM_PACKET_ARRIVED		0
#define COM_PACKET_SENT			1
#define COM_SIGNAL_NBR			2  

#define PDAPI_EVENT_PORT_ATTACHED		1
#define PDAPI_EVENT_PORT_DETACHED		2
#define PDAPI_EVENT_PROFILE_CHANGED		3
#define PDAPI_EVENT_NEW_CAPABILITY		4


typedef void(*type_EventCallBack) (int EventCode);

typedef enum
{
	USB_STATUS_NOT_CONNECTED = 0,
	USB_STATUS_CONNECTED
} USB_ConnectionStatus_t;

typedef enum
{
	PROFILE_LEGACY = 0,
	PROFILE_PTY,
	PROFILE_BC,
	PROFILE_QC,
	PROFILE_UC,
	PROFILE_PD
}PROFILE_TypeDef;

#define SUBTYPE_USB_DEFAULT		0

enum
{
	SUBTYPE_QC1,
	SUBTYPE_QC2,
	SUBTYPE_QC3
};

enum
{
	SUBTYPE_BC_SDP = 0,
	SUBTYPE_BC_CDP,
	SUBTYPE_BC_DCP,
};

enum
{
	SUBTYPE_UC_1_5A = 0,
	SUBTYPE_UC_3A
};

enum
{
	SUBTYPE_PD_FIX = 0,
	SUBTYPE_PD_BAT,
	SUBTYPE_PD_VAR,
	SUBTYPE_PD_APDO
};

enum
{
	SUBTYPE_PTY_APPLE_0_5A = 0,
	SUBTYPE_PTY_APPLE_1A,
	SUBTYPE_PTY_APPLE_2_1A,
	SUBTYPE_PTY_APPLE_2_4A
};

typedef struct ProfileInfo {
	UINT16 Index : 4;
	UINT16 Type : 4;
	UINT16 SubType : 3;
	UINT16 PDOIndex : 4;
	UINT16 bSelectable : 1;
}ProfileInfo_TypeDef;

typedef struct
{
	ProfileInfo_TypeDef 	Profile;
	UINT16 	MinVoltage;
	UINT16 	MaxVoltage;
	UINT16	MaxCurrent;
}USBPD_Object_TypeDef;

typedef struct
{
	BYTE	NumObjects;
	USBPD_Object_TypeDef Object[USBPD_MAX_NB_PDO];
}USBPD_Capabilities_TypeDef;


typedef enum
{
	ENFORCE_LIMITS = 0,
	ALLOW_20_PERCENT_OVERCURRENT,
	FORCE_LIMIT
} CurrentLimit_t;

typedef enum
{
	CALIB_INDEX_VBUS, 
	CALIB_INDEX_IBUS
}CalibrationChannel_t;


/* -------------------------------------------------------------------- */
/* -----------------------------  Tserial  ---------------------------- */
/* -------------------------------------------------------------------- */
class PDTester
{
	// -------------------------------------------------------- //
protected:
	Tserial_event *com;
	DWORD	NumPackets;
	char	ComRcvBuffer[4096];
	WORD	NumRcvBytes;
	WORD	NumRemaingBytes;
	UCHAR	RcvData[4096];
	WORD	RcvDataLen = 0;
	HANDLE	com_events[COM_SIGNAL_NBR];  // events to wait on
	type_EventCallBack	EventCallback;
	CRITICAL_SECTION	DeviceCritSection;
		
	// ++++++++++++++++++++++++++++++++++++++++++++++
	// .................. EXTERNAL VIEW .............
	// ++++++++++++++++++++++++++++++++++++++++++++++
public:
	UCHAR	HW_Ver;
	UCHAR	FW_Ver;
	bool	bLoopbackPortEnabled;
	bool	bEstimatePortVoltage;
	WORD	MaxSDPCurrent;
	CurrentLimit_t CurrentLimitType;
	WORD	MaxCurrent;
	WORD	CableResistance;

	PDTester();
	~PDTester();
	void	onSerialEvent(void *object, uint32 event);
	int		GetConnectedDevices(char *devices[MAX_NUM_TESTERS]);
	BOOL	Connect(char *port, type_EventCallBack event_callback);
	BOOL	GetDevInfo(UCHAR *HW_Ver, UCHAR *SW_Ver);
	BOOL	GetConfig(void);
	BOOL	SetConfig(void);
	BOOL	GetConnectionStatus(USB_ConnectionStatus_t* port_status, BYTE* profile_index, PROFILE_TypeDef* profile, BYTE* profile_subtype, UINT16* voltage, UINT16* max_current, UINT32* max_power);
	BOOL	GetCapabilities(USBPD_Capabilities_TypeDef *SourceCapabilities);
	BOOL	GetStatistics(UCHAR *temp, UINT16 *voltage, UINT16 *set_current, UINT16 *current, UINT16 *loopback_current);
	BOOL	SetVoltage(UCHAR index, UINT16 voltage);
	BOOL	SetLoad(UINT16 set_current);
	BOOL	SetDefaultVoltage(UINT16 voltage_mv);
	BOOL	SetDefaultProfile(UINT8 profile_index);
	BOOL	SetDefaultLoad(UINT16 current_ma);
	BOOL	SetCurrentLimit(void);
	BOOL	GetCalibrationData(CalibrationChannel_t channel, BOOL* isCalibrated, int* year, int* month, int* applied1, int* measured1, int* applied2, int* measured2);
	BOOL	SetCalibrationData(CalibrationChannel_t channel, int year, int month, int applied1, int measured1, int applied2, int measured2);
	BOOL	ResetCalibrationData(CalibrationChannel_t channel);
	void	Disconnect();

};
/* -------------------------------------------------------------------- */

#endif PASSMARK_PDTESTER_H

