
#define STRICT
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <conio.h>
#include <windows.h>

#include "Tserial_event.h"

#define SIG_POWER_DOWN     0
#define SIG_READER         1
#define SIG_READ_DONE      2    // data received has been read
#define SIG_WRITER         3
#define SIG_DATA_TO_TX     4    // data waiting to be sent
#define SIG_MODEM_EVENTS   5
#define SIG_MODEM_CHECKED  6

void Tserial_event_thread_start(void *arg);

typedef unsigned (WINAPI *PBEGINTHREADEX_THREADFUNC) (LPVOID lpThreadParameter);
typedef unsigned *PBEGINTHREADEX_THREADID;

/* ---------------------------------------------------------------------- */
/* ---------------------  Tserial_event_thread_start  ------------------- */
/* ---------------------------------------------------------------------- */
/**
This function is not part of the Tserial_event object. It is simply used
to start the thread from an external point of the object.
*/
void Tserial_event_thread_start(void *arg)
{
	class Tserial_event *serial_unit;

	serial_unit = (Tserial_event *)arg;

	if (serial_unit != 0)
		serial_unit->run();
}

/* -------------------------------------------------------------------- */
/* -------------------------    Tserial_event ------------------------- */
/* -------------------------------------------------------------------- */
Tserial_event::Tserial_event()
{
	int i;

	ready = false;
	parityMode = SERIAL_PARITY_NONE;
	port[0] = 0;
	rate = 0;
	threadid = 0;
	serial_handle = INVALID_HANDLE_VALUE;
	thread_handle = 0;
	owner = 0;
	tx_in_progress = 0;
	rx_in_progress = 0;
	max_rx_size = 1;
	tx_size = 0;
	received_size = 0;
	check_modem = false;

	manager = 0;

	/* -------------------------------------------------------------- */
	// creating Events for the different sources
	for (i = 0; i < SERIAL_SIGNAL_NBR; i++)
	{
		if ((i == SIG_READER) || (i == SIG_WRITER) || (i == SIG_MODEM_EVENTS) || (i == SIG_DATA_TO_TX))
			serial_events[i] = CreateEvent(NULL, TRUE, FALSE, NULL);  // Manual Reset
		else
			serial_events[i] = CreateEvent(NULL, FALSE, FALSE, NULL); // Auto reset
	}
}

/* -------------------------------------------------------------------- */
/* --------------------------    ~Tserial_event ----------------------- */
/* -------------------------------------------------------------------- */
Tserial_event::~Tserial_event()
{
	int i;

	if (thread_handle != 0)
		WaitForSingleObject(thread_handle, 2000);
	thread_handle = 0;

	/* -------------------------------------------------------- */
	for (i = 0; i < SERIAL_SIGNAL_NBR; i++)         // deleting the events
	{
		if (serial_events[i] != INVALID_HANDLE_VALUE)
			CloseHandle(serial_events[i]);
		serial_events[i] = INVALID_HANDLE_VALUE;
	}

	FT_Close(serial_handle); 

	serial_handle = INVALID_HANDLE_VALUE;
}
/* -------------------------------------------------------------------- */
/* --------------------------    disconnect   ------------------------- */
/* -------------------------------------------------------------------- */
void Tserial_event::disconnect(void)
{
	ready = false;
	SetEvent(serial_events[SIG_POWER_DOWN]);

	if (thread_handle != 0)
		WaitForSingleObject(thread_handle, 2000);
	thread_handle = 0;
}
/* -------------------------------------------------------------------- */
/* --------------------------    connect      ------------------------- */
/* -------------------------------------------------------------------- */
/**
Serial port, file and overlapped structures initialization
*/
bool	Tserial_event::connect(char *port)
{
	bool  ret = 1;
	int  i;
	FT_STATUS		ftStatus;
	COMMTIMEOUTS cto = { 0, 1, 10, 0, 0 };

	FT_Close(serial_handle);

	if (port != 0)
	{
		ZeroMemory(&ovReader, sizeof(ovReader));  // clearing the overlapped
		ZeroMemory(&ovWriter, sizeof(ovWriter));
		ZeroMemory(&ovWaitEvent, sizeof(ovWaitEvent));

		/* -------------------------------------------------------------------- */
		serial_handle = FT_W32_CreateFile((LPCTSTR)port, GENERIC_READ | GENERIC_WRITE, 0, 0,
			OPEN_EXISTING,
			FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED | FT_OPEN_BY_SERIAL_NUMBER,
			0);

		if (serial_handle == INVALID_HANDLE_VALUE)
			ret = 0;


		//ftStatus = FT_OpenEx(port, FT_OPEN_BY_SERIAL_NUMBER, &serial_handle);
		//if (ftStatus != FT_OK)
		//	ret = 0;
		// opening serial port

		ovReader.hEvent = serial_events[SIG_READER];
		ovWriter.hEvent = serial_events[SIG_WRITER];
		ovWaitEvent.hEvent = serial_events[SIG_MODEM_EVENTS];

		ftStatus = FT_SetBaudRate(serial_handle, 115200);
		if (ftStatus != FT_OK)
			ret = 0;

		ftStatus = FT_SetDataCharacteristics(serial_handle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE);
		if (ftStatus != FT_OK)
			ret = 0;

		ftStatus = FT_SetTimeouts(serial_handle, 10000, 10000);
		if (ftStatus != FT_OK)
			ret = 0;
	}
	else
		ret = 0;


	/* --------------------------------------------- */
	for (i = 0; i < SERIAL_SIGNAL_NBR; i++)
	{
		if (serial_events[i] == INVALID_HANDLE_VALUE)
			ret = 0;
	}

	/* --------------------------------------------- */
	if (!ret)
	{
		FT_Close(serial_handle);
	}
	else
	{
		// start thread
		thread_handle = (HANDLE)_beginthreadex(NULL, 0,
			(PBEGINTHREADEX_THREADFUNC)Tserial_event_thread_start,
			this, 0, &threadid);
	}

	/* --------------------------------------------- */
	return ret;
}
/* -------------------------------------------------------------------- */
/* ---------------------           setManager     --------------------- */
/* -------------------------------------------------------------------- */
void Tserial_event::setManager(type_myCallBack manager_arg, void* parent_class)
{
	manager = manager_arg;
	parent = parent_class;
}
/* -------------------------------------------------------------------- */
/* ---------------------           setRxSize      --------------------- */
/* -------------------------------------------------------------------- */
void Tserial_event::setRxSize(int size)
{
	max_rx_size = size;
	if (max_rx_size>SERIAL_MAX_RX)
		max_rx_size = SERIAL_MAX_RX;
}
/* -------------------------------------------------------------------- */
/* ---------------------           setManager     --------------------- */
/* -------------------------------------------------------------------- */
char *Tserial_event::getDataInBuffer(void)
{
	return(rxBuffer);
}
/* -------------------------------------------------------------------- */
/* ---------------------           setManager     --------------------- */
/* -------------------------------------------------------------------- */
int Tserial_event::getDataInSize(void)
{
	return(received_size);
}
/* -------------------------------------------------------------------- */
/* ---------------------           setManager     --------------------- */
/* -------------------------------------------------------------------- */
void Tserial_event::dataHasBeenRead(void)
{
	SetEvent(serial_events[SIG_READ_DONE]);
}
/* -------------------------------------------------------------------- */
/* --------------------------    sendData     ------------------------- */
/* -------------------------------------------------------------------- */
void Tserial_event::sendData(char *buffer, int size)
{
	if ((!tx_in_progress) && (size<SERIAL_MAX_TX) && (buffer != 0))
	{
		tx_in_progress = 1;
		memcpy(txBuffer, buffer, size);
		tx_size = size;
		SetEvent(serial_events[SIG_DATA_TO_TX]);
		// indicating data to be sent
	}
}
/* -------------------------------------------------------------------- */
/* --------------------------    OnEvent      ------------------------- */
/* -------------------------------------------------------------------- */
void Tserial_event::OnEvent(unsigned long events)
{
	unsigned long ModemStat;

	GetCommModemStatus(serial_handle, &ModemStat);

	if ((events & EV_RING) != 0)
	{
		if ((ModemStat &  MS_RING_ON) != 0)
		{
			if (manager != 0)
				manager((void*) this, parent, SERIAL_RING);
		}
	}

	if ((events & EV_RLSD) != 0)
	{
		if ((ModemStat &  MS_RLSD_ON) != 0)
		{
			if (manager != 0)
				manager((void*) this, parent, SERIAL_CD_ON);
		}
		else
		{
			if (manager != 0)
				manager((void*) this, parent, SERIAL_CD_OFF);
		}
	}
}
/* -------------------------------------------------------------------- */
/* --------------------------       run       ------------------------- */
/* -------------------------------------------------------------------- */

/**
this function is the main loop of the Tserial object. There is a
do while() loop executed until either an error or a PowerDown is
received.
this is not a busy wait since we use the WaitForMultipleObject function
*/

/* * /
#define DEBUG_EVENTS
/* */

void Tserial_event::run(void)
{
	bool          done;
	long          status;
	unsigned long read_nbr, result_nbr;
	char          success;

	ready = true;
	done = false;
	tx_in_progress = 0;
	rx_in_progress = 0;
	WaitCommEventInProgress = 0;

	if (manager != 0)
		manager(this, parent, SERIAL_CONNECTED);

	/* ----------------------------------------------------------- */
	while (!done)
	{
		/* ------------------------------------------------------------------ */
		/*                                                                    */
		/*                                                                    */
		/*                                                                    */
		/*                          Waiting  for signals                      */
		/*                                                                    */
		/*                                                                    */
		/*                                                                    */
		/* ------------------------------------------------------------------ */
		// Main wait function. Waiting for something to happen.
		// This may be either the completion of a Read or a Write or
		// the reception of modem events, Power Down, new Tx
		//
		status = WaitForMultipleObjects(SERIAL_SIGNAL_NBR, serial_events,
			FALSE, INFINITE);

		// processing answer to filter other failures
		status = status - WAIT_OBJECT_0;
		if ((status<0) || (status >= SERIAL_SIGNAL_NBR))
			done = true;   // error
		else
		{
			/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
			/* ++++++++++++++++++++ EVENT DISPATCHER ++++++++++++++++++ */
			/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
			switch (status)
			{
				/* ######################################################## */
			case SIG_POWER_DOWN:
				// receiving a POWER down signal. Stopping the thread
				done = true;
				break;
				/* ######################################################## */
				/* #                                                      # */
				/* #                                                      # */
				/* #                       RX                             # */
				/* #                                                      # */
				/* #                                                      # */
				/* ######################################################## */
			case SIG_READ_DONE:
				// previous reading is finished
				// I start a new one here
				if (!rx_in_progress)
				{
					// locking reading
					rx_in_progress = 1;

					// starting a new read

					success = (char)FT_W32_ReadFile(serial_handle, &rxBuffer, max_rx_size, &read_nbr, &ovReader);
					//if (!success)
					//	done = true;
				}
				break;
				/* ######################################################## */
			case SIG_READER:

				if (FT_W32_GetOverlappedResult(serial_handle, &ovReader,
					&result_nbr, FALSE))
				{
					// no error => OK
					// Read operation completed successfully
					ResetEvent(serial_events[SIG_READER]);
					// Write operation completed successfully
					received_size = result_nbr;
					rx_in_progress = 0; // read has ended
					// if incoming data, I process them
					if (manager != 0)
						manager(this, parent, SERIAL_DATA_ARRIVAL);
				}
				else
				{
					ResetEvent(serial_events[SIG_READER]);
					received_size = 0;
					rx_in_progress = 0;
				}
				break;
				/* ######################################################## */
				/* #                                                      # */
				/* #                                                      # */
				/* #                       TX                             # */
				/* #                                                      # */
				/* #                                                      # */
				/* ######################################################## */
			case SIG_DATA_TO_TX:
				// sending data to the port
				success = (char)FT_W32_WriteFile(serial_handle, txBuffer, tx_size,
					&result_nbr, &ovWriter);
				//if (!success)
				//	done = true;
				ResetEvent(serial_events[SIG_DATA_TO_TX]);
				break;
				/* ######################################################## */
			case SIG_WRITER:
				// WriteFile has terminated
				// checking the result of the operation
				if (FT_W32_GetOverlappedResult(serial_handle, &ovWriter,
					&result_nbr, FALSE))
				{
					// Write operation completed successfully
					ResetEvent(serial_events[SIG_WRITER]);
					// further write are now allowed
					tx_in_progress = 0;
					// telling it to the manager
					if (manager != 0)
						manager(this, parent, SERIAL_DATA_SENT);
				}
				else
				{
					ResetEvent(serial_events[SIG_WRITER]);
				}
				break;
			}
		}
	};

	// --------------------- Disconnecting ----------------

	ready = false;
	FT_Close(serial_handle);


	if (manager != 0)
		manager(this, parent, SERIAL_DISCONNECTED);
}
/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
